------------------------------------------------------------------------------
--
-- Filename:                aoi87.vhd
-- Inhalt:                  Modell eines AND-OR-INVERT-Komplexgatters,
--                          verwendet Typ "bit"
--
------------------------------------------------------------------------------
--
-- Autor:                   Bernhard Wunder 
-- Letzte Aenderung:        18.01.94
--
------------------------------------------------------------------------------

ENTITY aoi IS
    
    PORT (a1, a2, a3, b1, b2 : IN bit;
          y: OUT bit);                 -- implizite Signale

END aoi;

------------------------------------------------------------------------------
--
-- structural_1: Herkoemmliche Instantiierung, Konfiguration in aoi_config_1
--
------------------------------------------------------------------------------

ARCHITECTURE structural_1 OF aoi IS

  SIGNAL a_out, b_out, or_out : bit ;  -- interne Signale

  COMPONENT inv
    GENERIC (tpd_lh, tpd_hl : time := 0.8 ns) ;
    PORT (a : IN bit; y : OUT bit) ;
  END COMPONENT ;

  COMPONENT or2
    GENERIC (tpd_lh : time := 1.5 ns; tpd_hl : time := 1 ns) ;
    PORT (a,b : IN bit; y : OUT bit) ;
  END COMPONENT ;

  COMPONENT and2
    GENERIC (tpd_lh : time := 1 ns; tpd_hl : time := 1.5 ns) ;
    PORT (a,b : IN bit; y : OUT bit) ;
  END COMPONENT ;

  COMPONENT and3
    GENERIC (tpd_lh : time := 1 ns; tpd_hl : time := 1.8 ns) ;
    PORT (a,b,c : IN bit; y : OUT bit) ;
  END COMPONENT ;

BEGIN

  -- verschiedene Varianten von GENERIC und PORT MAP

  and_a : and3 GENERIC MAP (1.2 ns, 2.4 ns)
               PORT MAP (a1,a2,a3,a_out) ;       

  and_b : and2 GENERIC MAP (tpd_hl=>1.9 ns,tpd_lh=>1.1 ns)
               PORT MAP (b=>b1,y=>b_out,a=>b2) ;

  or_c  : or2  PORT MAP (a_out,y=>or_out,b=>b_out) ;
               -- Default Parameter: 1.5 bzw. 1.0 ns

  inv_d : inv  GENERIC MAP (0.7 ns) PORT MAP (or_out,y) ;
               -- tpd_hl verbleibt beim Defaultwert: 0.8 ns

END structural_1 ;

------------------------------------------------------------------------------

CONFIGURATION aoi_config_1 OF aoi IS

    FOR structural_1
        FOR inv_d : inv  USE ENTITY work.not1 (behavioral) ;
        END FOR ;
        FOR or_c  : or2  USE ENTITY work.or2  (behavioral) ;
        END FOR ;
        FOR and_b : and2 USE ENTITY work.and2 (behavioral) ;
        END FOR ;
        FOR and_a : and3 USE ENTITY work.and3 (behavioral) ;
        END FOR ;
    END FOR ;

END aoi_config_1 ;

------------------------------------------------------------------------------
--
-- structural_2 und structural_3: In aoi93.vhd
--
------------------------------------------------------------------------------

------------------------------------------------------------------------------
--
-- structural_4: Strukturale Architektur mit Bloecken (Hierarchieebenen);
--               Konfiguration in aoi_config_4
--
------------------------------------------------------------------------------

ARCHITECTURE structural_4 OF aoi IS

  SIGNAL a_out, b_out : bit ;       -- interne Signale

BEGIN

  and_stage : BLOCK

    COMPONENT and2
      PORT (a,b : IN bit; y : OUT bit) ;
    END COMPONENT ;

    COMPONENT and3
      PORT (a,b,c : IN bit; y : OUT bit) ;
    END COMPONENT ;
  
  BEGIN

    and_a : and3 PORT MAP (a1,a2,a3,a_out) ;
    and_b : and2 PORT MAP (b1,b2,b_out) ;

  END BLOCK and_stage;

-----------------------

  nor_stage : BLOCK

    PORT (aa,bb : IN bit; yy : OUT bit) ;
    PORT MAP (aa=>a_out, bb=>b_out, yy=>y) ;

    SIGNAL cc : bit;           -- block-internes Signal

    COMPONENT or2              -- nicht invertierernd !
      PORT (a,b : IN bit; y : OUT bit) ;
    END COMPONENT ;

  BEGIN

    or_c : or2 PORT MAP (a=>aa,b=>bb,y=>cc) ;

    yy <= NOT cc;              -- Invertierung
  
  END BLOCK nor_stage ;

END structural_4 ;

------------------------------------------------------------------------------  

CONFIGURATION aoi_config_4 OF aoi IS

    FOR structural_4
        FOR nor_stage
            FOR or_c : or2  USE ENTITY work.or2  (behavioral) ;
            END FOR ;
        END FOR;
  
        FOR and_stage
           FOR and_b : and2 USE ENTITY work.and2 (behavioral) ;
           END FOR ;
           FOR and_a : and3 USE ENTITY work.and3 (behavioral) ;
           END FOR ;
        END FOR;
    END FOR ;

END aoi_config_4 ;

------------------------------------------------------------------------------  

CONFIGURATION aoi_config_4a OF aoi IS

    FOR structural_4
        FOR nor_stage
            FOR or_c : or2  USE ENTITY work.or2  (behavioral) ;
            END FOR ;
        END FOR;
  
        FOR and_stage

-- hier: Verwendung eines AND3-Gatters in einem AND2-Sockel durch
-- Angabe von OPEN als ACTUAL; der Eingangsport a in der ENTITY
-- work.and3 muss daher einen Defaultwert besitzen!

           FOR and_b : and2 USE ENTITY work.and3 (behavioral)
               PORT MAP (a=>OPEN,b=>a,c=>b,y=>y) ;
           END FOR ;
           FOR and_a : and3 USE ENTITY work.and3 (behavioral) ;
           END FOR ;
        END FOR;
    END FOR ;

END aoi_config_4a ;

------------------------------------------------------------------------------
--
-- structural_5: Strukturale Architektur mit Default-Konfiguration der
--               Komponenten in aoi_config_6; Namen der COMPONENTs und
--               der Entities in work muessen dabei uebereinstimmen!
--
------------------------------------------------------------------------------

ARCHITECTURE structural_5 OF aoi IS

  SIGNAL a_out, b_out, or_out : bit ;  -- interne Signale

  COMPONENT not1
    GENERIC (tpd_lh, tpd_hl : time := 0.8 ns) ;
    PORT (a : IN bit; y : OUT bit) ;
  END COMPONENT ;

  COMPONENT or2
    GENERIC (tpd_lh : time := 1.5 ns; tpd_hl : time := 1 ns) ;
    PORT (a,b : IN bit; y : OUT bit) ;
  END COMPONENT ;

  COMPONENT and2
    GENERIC (tpd_lh : time := 1 ns; tpd_hl : time := 1.5 ns) ;
    PORT (a,b : IN bit; y : OUT bit) ;
  END COMPONENT ;

  COMPONENT and3
    GENERIC (tpd_lh : time := 1 ns; tpd_hl : time := 1.8 ns) ;
    PORT (a,b,c : IN bit; y : OUT bit) ;
  END COMPONENT ;

BEGIN

  and_a : and3 PORT MAP (a1,a2,a3,a_out) ;       

  and_b : and2 PORT MAP (b=>b1,y=>b_out,a=>b2) ;

  or_c  : or2  PORT MAP (a_out,y=>or_out,b=>b_out) ;

  inv_d : not1 PORT MAP (or_out,y) ;

END structural_5 ;

------------------------------------------------------------------------------  

CONFIGURATION aoi_config_5 OF aoi IS

    FOR structural_5

        -- Default-Configuration (keine Komponentenkonfigurationen)

    END FOR;

END aoi_config_5;

------------------------------------------------------------------------------
--
-- structural_6: Strukturale Architektur mit Konfiguration im Deklarations-
--               teil der Architektur selbst
--
------------------------------------------------------------------------------

ARCHITECTURE structural_6 OF aoi IS

  SIGNAL a_out, b_out, or_out: bit ;       -- interne Signale

  COMPONENT inv
    GENERIC (tpd_lh, tpd_hl : time := 0.8 ns) ;
    PORT (a : IN bit; y : OUT bit) ;
  END COMPONENT ;

  COMPONENT or2
    GENERIC (tpd_lh : time := 1.5 ns; tpd_hl : time := 1 ns) ;
    PORT (a,b : IN bit; y : OUT bit) ;
  END COMPONENT ;

  COMPONENT and2
    GENERIC (tpd_lh : time := 1 ns; tpd_hl : time := 1.5 ns) ;
    PORT (a,b : IN bit; y : OUT bit) ;
  END COMPONENT ;

  COMPONENT and3
    GENERIC (tpd_lh : time := 1 ns; tpd_hl : time := 1.8 ns) ;
    PORT (a,b,c : IN bit; y : OUT bit) ;
  END COMPONENT ;

  FOR ALL : inv  USE ENTITY work.not1 (behavioral) ;   -- Entity + Arch.
  FOR ALL : or2  USE ENTITY work.or2 (behavioral);     -- Entity + Arch.
  FOR ALL : and2 USE CONFIGURATION work.and2_config ;  -- Configuration
  FOR ALL : and3 USE CONFIGURATION work.and3_config ;

BEGIN

  and_a : and3 GENERIC MAP (1.2 ns, 2.4 ns)
               PORT MAP (a1,a2,a3,a_out) ;       

  and_b : and2 GENERIC MAP (tpd_hl=>1.9 ns,tpd_lh=>1.1 ns)
               PORT MAP (b=>b1,y=>b_out,a=>b2) ;

  or_c  : or2  PORT MAP (a_out,y=>or_out,b=>b_out) ;
               -- Default Parameter: 1.5 bzw. 1.0 ns

  inv_d : inv  GENERIC MAP (0.7 ns) PORT MAP (or_out,y) ;
               -- tpd_hl verbleibt beim Defaultwert: 0.8 ns

END structural_6 ;

------------------------------------------------------------------------------  

-- CONFIGURATION aoi_config_6 OF aoi IS      -- Nicht erforderlich !!!

------------------------------------------------------------------------------
--
-- structural_7: In aoi93.vhd
--
------------------------------------------------------------------------------
      
